#  Copyright (c) 2021 GradsFlow. All rights reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
from pathlib import Path

from gradsflow.core.data import Data
from gradsflow.data.image import image_dataset_from_directory

data_dir = Path.cwd()


# todo: remote dataset test
def test_image_dataset_from_directory():
    folder = f"{data_dir}/data/test-data-cat-dog-v0/cat-dog/"
    res1 = image_dataset_from_directory(folder, transform=True, ray_data=True)
    assert isinstance(res1, Data)
    res2 = image_dataset_from_directory(folder, transform=True, ray_data=False)
    assert isinstance(res2, Data)
