#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['gradsflow',
 'gradsflow.callbacks',
 'gradsflow.core',
 'gradsflow.data',
 'gradsflow.models',
 'gradsflow.tasks',
 'gradsflow.tasks.autoclassification',
 'gradsflow.tuner',
 'gradsflow.utility']

package_data = \
{'': ['*']}

install_requires = \
['accelerate>=0.5.0',
 'smart_open>=5.1',
 'lightning-flash[image,text]==0.5.0',
 'ray[default,tune]>=1.6',
 'rich',
 'loguru>=0.5']

extras_require = \
{'test': ['pytest', 'coverage']}

setup(name='gradsflow',
      version='0.0.6.post1',
      description='An open-source AutoML Library based on PyTorch',
      author='Aniket Maurya',
      author_email='aniket@gradsflow.com',
      url='https://github.com/gradsflow/gradsflow',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.7',
     )
